"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextAsset = exports.ImageAsset = exports.AudioAsset = exports.VideoAsset = void 0;
const constants_1 = require("../constants");
const uuid_1 = require("uuid");
const config_1 = require("./config");
const validateMaxSupport = (duration, maxDuration, attribute) => {
    if (!duration) {
        return 0;
    }
    else if (duration > maxDuration) {
        console.log(`${attribute}: ${duration} is greater than max supported: ${maxDuration}`);
    }
    return duration;
};
/**
 * The base MediaAsset class
 */
class MediaAsset {
    assetId;
    constructor(assetId) {
        this.assetId = assetId;
    }
}
/**
 * VideoAsset class
 */
class VideoAsset extends MediaAsset {
    start;
    end;
    /**
     * Initializes a AudioAsset instance
     * @param assetId - The id of the audio asset.
     * @param config - The configuration of the audio asset.
     * @returns A new AudioAsset instance.
     */
    constructor(assetId, config) {
        const { start = 0, end = null } = config || {};
        super(assetId);
        this.start = start;
        this.end = end;
    }
    toJSON() {
        return {
            assetId: this.assetId,
            start: this.start,
            end: this.end,
        };
    }
}
exports.VideoAsset = VideoAsset;
/**
 * AudioAsset class
 */
class AudioAsset extends MediaAsset {
    start;
    end;
    disableOtherTracks;
    fadeInDuration;
    fadeOutDuration;
    /**
     * Initializes a AudioAsset instance
     * @param assetId - The id of the audio asset.
     * @param config - The configuration of the audio asset.
     * @returns A new AudioAsset instance.
     */
    constructor(assetId, config) {
        const { start = 0, end = null, disableOtherTracks = true, fadeInDuration = 0, fadeOutDuration = 0, } = config || {};
        super(assetId);
        this.start = start;
        this.end = end;
        this.disableOtherTracks = disableOtherTracks;
        this.fadeInDuration = validateMaxSupport(fadeInDuration, constants_1.MaxSupported.fadeDuration, 'fadeInDuration');
        this.fadeOutDuration = validateMaxSupport(fadeOutDuration, constants_1.MaxSupported.fadeDuration, 'fadeOutDuration');
    }
    toJSON() {
        return {
            assetId: this.assetId,
            start: this.start,
            disableOtherTracks: this.disableOtherTracks,
            fadeInDuration: this.fadeInDuration,
            fadeOutDuration: this.fadeOutDuration,
            end: this.end,
        };
    }
}
exports.AudioAsset = AudioAsset;
class ImageAsset extends MediaAsset {
    width;
    height;
    x;
    y;
    duration;
    constructor(assetId, config) {
        const { width = 100, height = 100, x = 80, y = 20, duration = null, } = config || {};
        super(assetId);
        this.width = width;
        this.height = height;
        this.x = x;
        this.y = y;
        this.duration = duration;
    }
    toJSON() {
        return {
            assetId: this.assetId,
            width: this.width,
            height: this.height,
            x: this.x,
            y: this.y,
            duration: this.duration,
        };
    }
}
exports.ImageAsset = ImageAsset;
class TextAsset extends MediaAsset {
    text;
    duration;
    style;
    constructor(config) {
        const { text = '', style = {}, duration = null } = config || {};
        const assetId = `txt-${(0, uuid_1.v4)()}`;
        super(assetId);
        this.text = text;
        this.duration = duration;
        this.style = Object.assign({}, config_1.TextStyleDefaultValues, style);
    }
    toJSON() {
        return {
            assetId: this.assetId,
            text: this.text,
            style: { ...this.style },
            duration: this.duration,
        };
    }
}
exports.TextAsset = TextAsset;
