import type { CollectionBase, ICollection } from '../interfaces/core';
import { IndexType, SearchType } from '../types/search';
import type { FileUploadConfig, URLUploadConfig } from '../types/collection';
import { HttpClient } from '../utils/httpClient';
import { Video } from './video';
import { Audio } from './audio';
import { Image } from './image';
/**
 * The base VideoDB class
 * @remarks
 * The base class through which all videodb actions are possible
 */
export declare class Collection implements ICollection {
    #private;
    meta: CollectionBase;
    constructor(http: HttpClient, data: CollectionBase);
    /** * Get all videos from the collection
     * @returns A list of objects of the Video class
     * @throws an error if the request fails
     */
    getVideos: () => Promise<Video[]>;
    /**
     * Get all the information for a specific video
     * @param videoId - Unique ID of the video.
     * @returns An object of the Video class
     * @throws an error if the request fails
     */
    getVideo: (videoId: string) => Promise<Video>;
    /**
     *
     * @param videoId - Id of the video to be deleted
     * @returns A promise that resolves when delete is successful
     * @throws an error if the request fails
     */
    deleteVideo: (videoId: string) => Promise<import("../types/response").ResponseOf<Record<string, never>>>;
    /** * Get all audios from the collection
     * @returns A list of objects of the Audio class
     * @throws an error if the request fails
     */
    getAudios: () => Promise<Audio[]>;
    /**
     * Get all the information for a specific audio
     * @param audioId- Unique ID of the audio.
     * @returns An object of the Audio class
     * @throws an error if the request fails
     */
    getAudio: (audioId: string) => Promise<Audio>;
    /**
     *
     * @param audioId- Id of the audio to be deleted
     * @returns A promise that resolves when delete is successful
     * @throws an error if the request fails
     */
    deleteAudio: (audioId: string) => Promise<import("../types/response").ResponseOf<Record<string, never>>>;
    /** * Get all images from the collection
     * @returns A list of objects of the Image class
     * @throws an error if the request fails
     */
    getImages: () => Promise<Image[]>;
    /**
     * Get all the information for a specific image
     * @param imageId- Unique ID of the image.
     * @returns An object of the Image class
     * @throws an error if the request fails
     */
    getImage: (imageId: string) => Promise<Image>;
    /**
     *
     * @param imageId- Id of the image to be deleted
     * @returns A promise that resolves when delete is successful
     * @throws an error if the request fails
     */
    deleteImage: (imageId: string) => Promise<import("../types/response").ResponseOf<Record<string, never>>>;
    /**
     * @param filePath - absolute path to a file
     * @param callbackUrl- [optional] A url that will be called once upload is finished
     * @param name - [optional] Name of the file
     * @param description - [optional] Description of the file
     *
     * @see
     * Providing a callbackUrl will return undefined and not providing one
     * will return a Job object (TODO: Implement proper type for this condition)
     */
    uploadFile: (data: FileUploadConfig) => Promise<import("..").UploadJob | undefined>;
    /**
     * @param URL - URL of the hosted file
     * @param callbackUrl- [optional] A url that will be called once upload is finished
     * @param name - [optional] Name of the file
     * @param description - [optional] Description of the file
     *
     * @see
     * Providing a callbackUrl will return undefined and not providing one
     * will return a Job object (TODO: Implement proper type for this condition)
     */
    uploadURL: (data: URLUploadConfig) => Promise<import("..").UploadJob | undefined>;
    /**
     * @param query - Search query
     * @param searchType - [optional] Type of search to be performed
     * @param resultThreshold - [optional] Result Threshold
     * @param scoreThreshold - [optional] Score Threshold
     */
    search: (query: string, searchType?: SearchType, indexType?: IndexType, resultThreshold?: number, scoreThreshold?: number) => Promise<import("..").SearchResult>;
}
