"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const upload_1 = require("../utils/upload");
const config_1 = require("../core/config");
const search_1 = require("./search");
const video_1 = require("./video");
const audio_1 = require("./audio");
const image_1 = require("./image");
const error_1 = require("../utils/error");
const { video, audio, image } = constants_1.ApiPath;
/**
 * The base VideoDB class
 * @remarks
 * The base class through which all videodb actions are possible
 */
class Collection {
    meta;
    #vhttp;
    constructor(http, data) {
        this.meta = data;
        this.#vhttp = http;
    }
    /** * Get all videos from the collection
     * @returns A list of objects of the Video class
     * @throws an error if the request fails
     */
    getVideos = async () => {
        const res = await this.#vhttp.get([video], {
            params: { collection_id: this.meta.id },
        });
        const videos = res.data.videos;
        return videos.map(vid => {
            const data = (0, utils_1.fromSnakeToCamel)(vid);
            return new video_1.Video(this.#vhttp, data);
        });
    };
    /**
     * Get all the information for a specific video
     * @param videoId - Unique ID of the video.
     * @returns An object of the Video class
     * @throws an error if the request fails
     */
    getVideo = async (videoId) => {
        if (!videoId.trim()) {
            throw new error_1.VideodbError('Video ID cannot be empty');
        }
        const res = await this.#vhttp.get([video, videoId], {
            params: { collection_id: this.meta.id },
        });
        const data = (0, utils_1.fromSnakeToCamel)(res.data);
        return new video_1.Video(this.#vhttp, data);
    };
    /**
     *
     * @param videoId - Id of the video to be deleted
     * @returns A promise that resolves when delete is successful
     * @throws an error if the request fails
     */
    deleteVideo = async (videoId) => {
        if (!videoId.trim()) {
            throw new error_1.VideodbError('Video ID cannot be empty');
        }
        return await this.#vhttp.delete([video, videoId], {
            params: { collection_id: this.meta.id },
        });
    };
    /** * Get all audios from the collection
     * @returns A list of objects of the Audio class
     * @throws an error if the request fails
     */
    getAudios = async () => {
        const res = await this.#vhttp.get([audio], {
            params: { collection_id: this.meta.id },
        });
        const audios = res.data.audios;
        return audios.map(audio => {
            const data = (0, utils_1.fromSnakeToCamel)(audio);
            return new audio_1.Audio(this.#vhttp, data);
        });
    };
    /**
     * Get all the information for a specific audio
     * @param audioId- Unique ID of the audio.
     * @returns An object of the Audio class
     * @throws an error if the request fails
     */
    getAudio = async (audioId) => {
        if (!audioId.trim()) {
            throw new error_1.VideodbError('Audio ID cannot be empty');
        }
        const res = await this.#vhttp.get([audio, audioId], {
            params: { collection_id: this.meta.id },
        });
        const data = (0, utils_1.fromSnakeToCamel)(res.data);
        return new audio_1.Audio(this.#vhttp, data);
    };
    /**
     *
     * @param audioId- Id of the audio to be deleted
     * @returns A promise that resolves when delete is successful
     * @throws an error if the request fails
     */
    deleteAudio = async (audioId) => {
        if (!audioId.trim()) {
            throw new error_1.VideodbError('Audio ID cannot be empty');
        }
        return await this.#vhttp.delete([audio, audioId], {
            params: { collection_id: this.meta.id },
        });
    };
    /** * Get all images from the collection
     * @returns A list of objects of the Image class
     * @throws an error if the request fails
     */
    getImages = async () => {
        const res = await this.#vhttp.get([image], {
            params: { collection_id: this.meta.id },
        });
        const images = res.data.images;
        return images.map(audio => {
            const data = (0, utils_1.fromSnakeToCamel)(audio);
            return new image_1.Image(this.#vhttp, data);
        });
    };
    /**
     * Get all the information for a specific image
     * @param imageId- Unique ID of the image.
     * @returns An object of the Image class
     * @throws an error if the request fails
     */
    getImage = async (imageId) => {
        if (!imageId.trim()) {
            throw new error_1.VideodbError('Image ID cannot be empty');
        }
        const res = await this.#vhttp.get([image, imageId], {
            params: { collection_id: this.meta.id },
        });
        const data = (0, utils_1.fromSnakeToCamel)(res.data);
        return new image_1.Image(this.#vhttp, data);
    };
    /**
     *
     * @param imageId- Id of the image to be deleted
     * @returns A promise that resolves when delete is successful
     * @throws an error if the request fails
     */
    deleteImage = async (imageId) => {
        if (!imageId.trim()) {
            throw new error_1.VideodbError('Image ID cannot be empty');
        }
        return await this.#vhttp.delete([image, imageId], {
            params: { collection_id: this.meta.id },
        });
    };
    /**
     * @param filePath - absolute path to a file
     * @param callbackUrl- [optional] A url that will be called once upload is finished
     * @param name - [optional] Name of the file
     * @param description - [optional] Description of the file
     *
     * @see
     * Providing a callbackUrl will return undefined and not providing one
     * will return a Job object (TODO: Implement proper type for this condition)
     */
    uploadFile = async (data) => {
        return (0, upload_1.uploadToServer)(this.#vhttp, this.meta.id, data);
    };
    /**
     * @param URL - URL of the hosted file
     * @param callbackUrl- [optional] A url that will be called once upload is finished
     * @param name - [optional] Name of the file
     * @param description - [optional] Description of the file
     *
     * @see
     * Providing a callbackUrl will return undefined and not providing one
     * will return a Job object (TODO: Implement proper type for this condition)
     */
    uploadURL = async (data) => {
        return (0, upload_1.uploadToServer)(this.#vhttp, this.meta.id, data);
    };
    /**
     * @param query - Search query
     * @param searchType - [optional] Type of search to be performed
     * @param resultThreshold - [optional] Result Threshold
     * @param scoreThreshold - [optional] Score Threshold
     */
    search = async (query, searchType, indexType, resultThreshold, scoreThreshold) => {
        const s = new search_1.SearchFactory(this.#vhttp);
        const searchFunc = s.getSearch(searchType ?? config_1.DefaultSearchType);
        const results = await searchFunc.searchInsideCollection({
            collectionId: this.meta.id,
            query: query,
            searchType: searchType ?? config_1.DefaultSearchType,
            indexType: indexType ?? config_1.DefaultIndexType,
            resultThreshold: resultThreshold,
            scoreThreshold: scoreThreshold,
        });
        return results;
    };
}
exports.Collection = Collection;
