import type { IVideo, VideoBase } from '../interfaces/core';
import type { Timeline, Transcript } from '../types/video';
import { HttpClient } from '../utils/httpClient';
import { IndexJob, TranscriptJob } from '../utils/job';
import { IndexSceneConfig, SubtitleStyleProps } from '../types/config';
import { SearchType, IndexType } from '../types/search';
import { SceneIndexRecords, SceneIndexes } from '../types';
/**
 * The base Video class
 * @remarks
 * Use this to initialize a video stored in videoDB
 */
export declare class Video implements IVideo {
    #private;
    meta: VideoBase;
    transcript?: Transcript;
    /**
     * Initializes a videoDB Instance
     * @param http - HttpClient object
     * @param data - Data needed to initialize a video instance
     */
    constructor(http: HttpClient, data: VideoBase);
    /**
     * @param query - Search query
     * @param searchType- [optional] Type of search to be performed
     * @param indexType- [optional] Index Type
     * @param resultThreshold - [optional] Result Threshold
     * @param scoreThreshold - [optional] Score Threshold
     */
    search: (query: string, searchType?: SearchType, indexType?: IndexType, resultThreshold?: number, scoreThreshold?: number) => Promise<import("..").SearchResult>;
    /**
     * Returns an empty promise that resolves when the video is deleted
     * @returns A promise that resolves when delete is successful
     * @throws an InvalidRequestError if the request fails
     */
    delete: () => Promise<import("../types/response").ResponseOf<Record<string, never>>>;
    /**
     * Generates a new streaming URL with the given timeline.
     * @param timeline - Of the format [[start, end], [start, end]...]
     * @returns a streaming URL
     */
    generateStream: (timeline?: Timeline) => Promise<string>;
    /**
     * Gets the thumbnail of the video or generates one if it doesn't exist.
     * @returns An awaited URL to the video's thumbnail
     */
    generateThumbnail: () => Promise<string>;
    /**
     * Fetches the transcript of the video if it exists, generates one
     * if it doesn't.
     * @param forceCreate - Forces transcript generation even if it exists
     * @returns A promise of -
     * - If the transcript exists, an object of the type Transcript
     * - If it doesn't, an instance of TranscriptJob which can be used
     *   to start transcript generation.
     */
    getTranscript: (forceCreate?: boolean) => Transcript | TranscriptJob;
    /**
     * Indexs the video semantically
     * @returns an awaited boolean signifying whether the process
     * was successful or not
     */
    indexSpokenWords: () => IndexJob;
    /**
     * Indexs the video with scenes
     * @returns an awaited boolean signifying whether the process
     * was successful or not
     */
    indexScenes: (config?: Partial<IndexSceneConfig>) => Promise<string | undefined>;
    listSceneIndex: () => Promise<SceneIndexes>;
    getSceneIndex: (sceneIndexId: string) => Promise<SceneIndexRecords>;
    deleteSceneIndex: (sceneIndexId: string) => Promise<import("../types/response").ResponseOf<unknown>>;
    /**
     * Overlays subtitles on top of a video
     * @returns an awaited stream url for subtitled overlayed video
     *
     */
    addSubtitle: (config?: Partial<SubtitleStyleProps>) => Promise<string>;
    /**
     * Generates a new playable stream URL with the given timeline.
     * @returns a URL that can be opened in browser
     */
    play: () => string;
}
