"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Video = void 0;
const index_1 = require("./../utils/index");
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const config_1 = require("../core/config");
const job_1 = require("../utils/job");
const search_1 = require("./search");
const { video, stream, thumbnail, workflow, index, scene, scenes } = constants_1.ApiPath;
/**
 * The base Video class
 * @remarks
 * Use this to initialize a video stored in videoDB
 */
class Video {
    meta;
    transcript;
    #vhttp;
    /**
     * Initializes a videoDB Instance
     * @param http - HttpClient object
     * @param data - Data needed to initialize a video instance
     */
    constructor(http, data) {
        this.meta = data;
        this.#vhttp = http;
    }
    /**
     * @param query - Search query
     * @param searchType- [optional] Type of search to be performed
     * @param indexType- [optional] Index Type
     * @param resultThreshold - [optional] Result Threshold
     * @param scoreThreshold - [optional] Score Threshold
     */
    search = async (query, searchType, indexType, resultThreshold, scoreThreshold) => {
        const s = new search_1.SearchFactory(this.#vhttp);
        const searchFunc = s.getSearch(searchType ?? config_1.DefaultSearchType);
        const results = await searchFunc.searchInsideVideo({
            videoId: this.meta.id,
            query: query,
            searchType: searchType ?? config_1.DefaultSearchType,
            indexType: indexType ?? config_1.DefaultIndexType,
            resultThreshold: resultThreshold,
            scoreThreshold: scoreThreshold,
        });
        return results;
    };
    /**
     * Returns an empty promise that resolves when the video is deleted
     * @returns A promise that resolves when delete is successful
     * @throws an InvalidRequestError if the request fails
     */
    delete = async () => {
        return await this.#vhttp.delete([
            video,
            this.meta.id,
        ]);
    };
    /**
     * Generates a new streaming URL with the given timeline.
     * @param timeline - Of the format [[start, end], [start, end]...]
     * @returns a streaming URL
     */
    generateStream = async (timeline) => {
        if (!timeline && this.meta.streamUrl) {
            return this.meta.streamUrl;
        }
        const body = {
            length: Number(this.meta.length),
        };
        if (timeline)
            body.timeline = timeline;
        const res = await this.#vhttp.post([video, this.meta.id, stream], body);
        return res.data.stream_url;
    };
    /**
     * Gets the thumbnail of the video or generates one if it doesn't exist.
     * @returns An awaited URL to the video's thumbnail
     */
    generateThumbnail = async () => {
        if (this.meta.thumbnail)
            return this.meta.thumbnail;
        const res = await this.#vhttp.get([
            video,
            this.meta.id,
            thumbnail,
        ]);
        this.meta.thumbnail = res.data.thumbnail;
        return res.data.thumbnail;
    };
    /**
     * Fetches the transcript of the video if it exists, generates one
     * if it doesn't.
     * @param forceCreate - Forces transcript generation even if it exists
     * @returns A promise of -
     * - If the transcript exists, an object of the type Transcript
     * - If it doesn't, an instance of TranscriptJob which can be used
     *   to start transcript generation.
     */
    getTranscript = (forceCreate = false) => {
        if (this.transcript && !forceCreate)
            return this.transcript;
        const job = new job_1.TranscriptJob(this.#vhttp, this.meta.id, forceCreate);
        return job;
    };
    /**
     * Indexs the video semantically
     * @returns an awaited boolean signifying whether the process
     * was successful or not
     */
    indexSpokenWords = () => {
        const indexJob = new job_1.IndexJob(this.#vhttp, this.meta.id, config_1.IndexTypeValues.spoken);
        return indexJob;
    };
    /**
     * Indexs the video with scenes
     * @returns an awaited boolean signifying whether the process
     * was successful or not
     */
    indexScenes = async (config = {}) => {
        const defaultConfig = {
            extractionType: config_1.SceneExtractionType.shotBased,
            extractionConfig: {},
        };
        const indexScenesPayload = (0, utils_1.fromCamelToSnake)(Object.assign({}, defaultConfig, config));
        const res = await this.#vhttp.post([video, this.meta.id, index, scene], indexScenesPayload);
        if (res.data) {
            return res.data.scene_index_id;
        }
    };
    listSceneIndex = async () => {
        const res = await this.#vhttp.get([
            video,
            this.meta.id,
            index,
            scene,
        ]);
        const transformed = (0, index_1.fromSnakeToCamel)({
            array: res.data.scene_indexes,
        }).array;
        return transformed;
    };
    getSceneIndex = async (sceneIndexId) => {
        const sceneIndexJob = new job_1.SceneIndexJob(this.#vhttp, this.meta.id, sceneIndexId);
        return new Promise((resolve, reject) => {
            sceneIndexJob.on('success', data => {
                resolve(data);
            });
            sceneIndexJob.on('error', err => {
                reject(err);
            });
            sceneIndexJob
                .start()
                .then(() => { })
                .catch(err => {
                reject(err);
            });
        });
    };
    deleteSceneIndex = async (sceneIndexId) => {
        const res = await this.#vhttp.delete([
            video,
            this.meta.id,
            index,
            scene,
            sceneIndexId,
        ]);
        return res;
    };
    /**
     * Overlays subtitles on top of a video
     * @returns an awaited stream url for subtitled overlayed video
     *
     */
    addSubtitle = async (config) => {
        const subtitlePayload = (0, utils_1.fromCamelToSnake)({
            type: constants_1.Workflows.addSubtitles,
            subtitleStyle: { ...config },
        });
        const res = await this.#vhttp.post([video, this.meta.id, workflow], subtitlePayload);
        return res.data.stream_url;
    };
    /**
     * Generates a new playable stream URL with the given timeline.
     * @returns a URL that can be opened in browser
     */
    play = () => {
        return (0, utils_1.playStream)(this.meta.streamUrl);
    };
}
exports.Video = Video;
