import { Video } from '../core/video';
import { Audio } from '../core/audio';
import { Image } from '../core/image';
import type { SyncUploadConfig } from '../types/collection';
import type { MediaBase, SceneIndexRecords } from '../types/index';
import type { IndexConfig } from '../types/config';
import type { IndexType } from '../types/search';
import type { NoDataResponse, TranscriptResponse, MediaResponse, GetSceneIndexResponse } from '../types/response';
import type { JobErrorCallback, JobSuccessCallback } from '../types/utils';
import type { Transcript } from '../types/video';
import { HttpClient } from './httpClient';
import { IndexSceneConfig } from '../types/config';
/**
 * Base Job class used to create different kinds of jobs
 * @remarks
 * Jobs are used for long running tasks where a simple
 * async call would take too long causing a timeout.
 *
 * @see
 * This class accepts 3 type params
 * - ApiResponse: The response recieved from the API on calling
 */
export declare abstract class Job<ApiResponse extends object, SdkBase, FinalReturn = SdkBase> {
    private _on;
    private readonly _delayMultiplier;
    private readonly _maxDelay;
    private currentDelaySeconds;
    protected vhttp: HttpClient;
    protected convertResponseToCamelCase: boolean;
    protected jobTitle: string;
    /**
     * @param http - HttpClient object
     */
    constructor(http: HttpClient);
    protected abstract beforeSuccess: (data: SdkBase) => FinalReturn;
    abstract start: () => Promise<void>;
    on(option: 'success', method: JobSuccessCallback<FinalReturn>): void;
    on(option: 'error', method: JobErrorCallback): void;
    private _incrementDelay;
    protected _handleError: (err: unknown) => void;
    protected _handleSuccess: (data: ApiResponse) => void;
    /**
     * Initiates a backoff-like system where we check the status
     * of the job in an exponentially increasing interval.
     * @param http - HttpClient instance
     * @param callbackUrl - URL sent by the server to check status
     *
     * @returns NOTHING. Do not await this function. This will call the
     * success or error listener depending on the status.
     */
    protected _initiateBackoff: (callbackUrl: string) => Promise<void>;
}
/**
 * TranscriptJob is used to initalize a new trancsript generation call.
 *
 * @remarks
 * Uses the base Job class to implement a backoff to get the transcript
 */
export declare class TranscriptJob extends Job<TranscriptResponse, Transcript> {
    videoId: string;
    force: boolean;
    constructor(http: HttpClient, videoId: string, force?: boolean);
    /**
     * If the transcript exists, it immediately calls
     * the success listener. If it doesn't exist, it
     * initiates a backoff.
     */
    start: () => Promise<void>;
    protected beforeSuccess: (data: Transcript) => Transcript;
}
/**
 * UploadJob is used to initalize a new video upload.
 *
 * @remarks
 * Uses the base Job class to implement a backoff to get the uploaded video data.
 */
export declare class UploadJob extends Job<MediaResponse, MediaBase, Video | Audio | Image> {
    uploadData: SyncUploadConfig;
    collectionId: string;
    constructor(data: SyncUploadConfig, collectionId: string, http: HttpClient);
    /**
     * Fetches the callbackURL from the server and initiates a backoff
     */
    start: () => Promise<void>;
    /**
     * Initializes a new video object with the returned data
     * @param data - Media data returned from the API and converted to camelCase
     * @returns a new Video object
     */
    protected beforeSuccess: (data: MediaBase) => Video | Audio | Image;
}
export declare class IndexJob extends Job<NoDataResponse, NoDataResponse> {
    videoId: string;
    indexConfig: IndexConfig;
    constructor(http: HttpClient, videoId: string, indexType: IndexType, additionalConfig?: IndexSceneConfig);
    /**
     * Initiates a Transcript Job.
     * On sucess, it calls the index endpoint
     */
    start: () => Promise<void>;
    protected beforeSuccess: (data: NoDataResponse) => {
        success: true;
        message?: undefined;
    } | {
        success: false | undefined;
        message: string | undefined;
    };
}
/**
 * SceneIndexJob is used to initalize a new video upload.
 *
 * @remarks
 * Uses the base Job class to implement a backoff to get the uploaded video data.
 */
export declare class SceneIndexJob extends Job<GetSceneIndexResponse, GetSceneIndexResponse, SceneIndexRecords> {
    videoId: string;
    sceneIndexId: string;
    constructor(http: HttpClient, videoId: string, sceneIndexId: string);
    /**
     * Fetches the callbackURL from the server and initiates a backoff
     */
    start: () => Promise<void>;
    /**
     * Initializes a new video object with the returned data
     * @param data - Media data returned from the API and converted to camelCase
     * @returns a new Video object
     */
    protected beforeSuccess: (data: GetSceneIndexResponse) => SceneIndexRecords;
}
