"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SceneIndexJob = exports.IndexJob = exports.UploadJob = exports.TranscriptJob = exports.Job = void 0;
const constants_1 = require("../constants");
const video_1 = require("../core/video");
const audio_1 = require("../core/audio");
const image_1 = require("../core/image");
const _1 = require(".");
const error_1 = require("./error");
const config_1 = require("../core/config");
const { in_progress, processing } = constants_1.ResponseStatus;
const { video, transcription, collection, upload, index, scene, scenes } = constants_1.ApiPath;
/**
 * Base Job class used to create different kinds of jobs
 * @remarks
 * Jobs are used for long running tasks where a simple
 * async call would take too long causing a timeout.
 *
 * @see
 * This class accepts 3 type params
 * - ApiResponse: The response recieved from the API on calling
 */
class Job {
    _on = {};
    _delayMultiplier = 2;
    _maxDelay = 1000000;
    currentDelaySeconds = 2000;
    vhttp;
    convertResponseToCamelCase = true;
    jobTitle;
    /**
     * @param http - HttpClient object
     */
    constructor(http) {
        this.vhttp = http;
        this.jobTitle = 'Job';
    }
    on(option, method) {
        if (option === 'success') {
            this._on[option] = method;
        }
        else if (option === 'error') {
            this._on[option] = method;
        }
    }
    _incrementDelay() {
        this.currentDelaySeconds = this._delayMultiplier * this.currentDelaySeconds;
    }
    _handleError = (err) => {
        if (this._on.error) {
            if (err instanceof error_1.AuthenticationError ||
                err instanceof error_1.InvalidRequestError ||
                err instanceof error_1.VideodbError)
                this._on.error(err);
            else {
                this._on.error(new error_1.VideodbError('Unknown Error', err));
            }
        }
        else {
            console.error('Unregistered Job Error', err);
        }
    };
    _handleSuccess = (data) => {
        const transformedResponse = (0, _1.fromSnakeToCamel)(data);
        const finalData = this.beforeSuccess(transformedResponse);
        if (this._on.success) {
            this._on.success(finalData);
        }
        else {
            console.log(`${this.jobTitle} Completed but success listener wasn't registered`, data);
        }
    };
    /**
     * Initiates a backoff-like system where we check the status
     * of the job in an exponentially increasing interval.
     * @param http - HttpClient instance
     * @param callbackUrl - URL sent by the server to check status
     *
     * @returns NOTHING. Do not await this function. This will call the
     * success or error listener depending on the status.
     */
    _initiateBackoff = async (callbackUrl) => {
        try {
            const res = await this.vhttp.get([callbackUrl]);
            if (res.status === in_progress || res.status === processing) {
                if (this.currentDelaySeconds >= this._maxDelay) {
                    throw new error_1.VideodbError('Job timed out');
                }
                setTimeout(() => {
                    void this._initiateBackoff(callbackUrl);
                    this._incrementDelay();
                }, this.currentDelaySeconds);
            }
            else {
                // If an error hasn't been thrown unitl now, the job has succeeded
                // TODO: remove the ignore comment after server update
                // @ts-ignore
                if ('response' in res && res.response) {
                    if (res.response.success === false) {
                        throw new error_1.VideodbError(res.response.message);
                    }
                    this._handleSuccess(res.response.data);
                }
                else {
                    this._handleSuccess(res.data);
                }
            }
        }
        catch (err) {
            this._handleError(err);
        }
    };
}
exports.Job = Job;
/**
 * TranscriptJob is used to initalize a new trancsript generation call.
 *
 * @remarks
 * Uses the base Job class to implement a backoff to get the transcript
 */
class TranscriptJob extends Job {
    videoId;
    force;
    constructor(http, videoId, force = false) {
        super(http);
        this.videoId = videoId;
        this.force = force;
        this.jobTitle = 'Transcript Job';
    }
    /**
     * If the transcript exists, it immediately calls
     * the success listener. If it doesn't exist, it
     * initiates a backoff.
     */
    start = async () => {
        try {
            const res = await this.vhttp.get([
                video,
                this.videoId,
                transcription,
                `?force=${String(this.force)}`,
            ]);
            if ('output_url' in res.data) {
                void this._initiateBackoff(res.data.output_url);
            }
            else {
                this._handleSuccess(res.data);
            }
        }
        catch (err) {
            this._handleError(err);
        }
    };
    // Transcript job doesn't need a beforeSuccess call so simply returns the same data
    beforeSuccess = (data) => data;
}
exports.TranscriptJob = TranscriptJob;
/**
 * UploadJob is used to initalize a new video upload.
 *
 * @remarks
 * Uses the base Job class to implement a backoff to get the uploaded video data.
 */
class UploadJob extends Job {
    uploadData;
    collectionId;
    constructor(data, collectionId, http) {
        super(http);
        this.uploadData = data;
        this.collectionId = collectionId;
        this.jobTitle = 'Upload Job';
    }
    /**
     * Fetches the callbackURL from the server and initiates a backoff
     */
    start = async () => {
        try {
            const res = await this.vhttp.post([collection, this.collectionId, upload], this.uploadData);
            void this._initiateBackoff(res.data.output_url);
        }
        catch (err) {
            this._handleError(err);
        }
    };
    /**
     * Initializes a new video object with the returned data
     * @param data - Media data returned from the API and converted to camelCase
     * @returns a new Video object
     */
    beforeSuccess = (data) => {
        const mediaId = data.id;
        if (mediaId.startsWith('img-')) {
            return new image_1.Image(this.vhttp, data);
        }
        else if (mediaId.startsWith('a-')) {
            return new audio_1.Audio(this.vhttp, data);
        }
        return new video_1.Video(this.vhttp, data);
    };
}
exports.UploadJob = UploadJob;
class IndexJob extends Job {
    videoId;
    indexConfig;
    constructor(http, videoId, indexType, additionalConfig = {}) {
        super(http);
        this.videoId = videoId;
        this.indexConfig = {
            indexType: indexType,
            ...additionalConfig,
        };
        this.jobTitle = 'Index Job';
    }
    /**
     * Initiates a Transcript Job.
     * On sucess, it calls the index endpoint
     */
    start = async () => {
        if (this.indexConfig.indexType === config_1.IndexTypeValues.spoken) {
            const transcriptJob = new TranscriptJob(this.vhttp, this.videoId);
            const reqData = (0, _1.fromCamelToSnake)(this.indexConfig);
            transcriptJob.on('success', async () => {
                try {
                    const res = await this.vhttp.post([video, this.videoId, index], reqData);
                    this._handleSuccess(res);
                }
                catch (err) {
                    this._handleError(err);
                }
            });
            transcriptJob.on('error', err => {
                this._handleError(err);
            });
            await transcriptJob.start();
        }
    };
    beforeSuccess = (data) => {
        if (data.success) {
            return {
                success: data.success,
            };
        }
        else {
            return {
                success: data.success,
                message: data.message,
            };
        }
    };
}
exports.IndexJob = IndexJob;
/**
 * SceneIndexJob is used to initalize a new video upload.
 *
 * @remarks
 * Uses the base Job class to implement a backoff to get the uploaded video data.
 */
class SceneIndexJob extends Job {
    videoId;
    sceneIndexId;
    constructor(http, videoId, sceneIndexId) {
        super(http);
        this.videoId = videoId;
        this.sceneIndexId = sceneIndexId;
        this.jobTitle = 'Scene Index Job';
    }
    /**
     * Fetches the callbackURL from the server and initiates a backoff
     */
    start = async () => {
        try {
            const res = await this.vhttp.get([
                video,
                this.videoId,
                index,
                scene,
                this.sceneIndexId,
            ]);
            if (res.status === processing) {
                void this._initiateBackoff(res.data.output_url);
            }
            else {
                // @ts-ignore
                this._handleSuccess(res.data);
            }
        }
        catch (err) {
            this._handleError(err);
        }
    };
    /**
     * Initializes a new video object with the returned data
     * @param data - Media data returned from the API and converted to camelCase
     * @returns a new Video object
     */
    beforeSuccess = (data) => {
        return data.sceneIndexRecords;
    };
}
exports.SceneIndexJob = SceneIndexJob;
